AVG_COLUMNS = {
    "Date": "date",
    "Time": "time",
    "AirTemperature": "air_temperature",
    "Radiation": "radiation",
    "CO2": "carbon_dioxide",
    "Precipitation": "precipitation",
    "WS": "wind_velocity",
    "WD": "wind_direction",
    "Humidity": "humidity",
    "UBat": "battery",
}

EXT_COLUMNS = {
    "Date": "date",
    "Time": "time",
    "AirTemperature MIN": "air_temperature",
    "AirTemperature MAX": "air_temperature",
    "Radiation MIN": "radiation",
    "Radiation MAX": "radiation",
    "CO2 MIN": "carbon_dioxide",
    "CO2 MAX": "carbon_dioxide",
    "WS MIN": "wind_velocity",
    "WS MAX gust": "wind_velocity",
    "WD MIN": "wind_direction",
    "WD MAX gust": "wind_direction",
    "Humidity MIN": "humidity",
    "Humidity MAX": "humidity",
    "UBat MIN": "battery",
    "UBat MAX": "battery",
}

AGG_DICT = {
    "AirTemperature": "mean",
    "AirTemperature MIN": "mean",
    "AirTemperature MAX": "mean",
    "Precipitation": "sum",
    "Humidity": "mean",
    "Humidity MIN": "mean",
    "Humidity MAX": "mean",
    "Radiation": "sum",
    "Radiation MIN": "sum",
    "Radiation MAX": "sum",
    "CO2": "mean",
    "CO2 MIN": "mean",
    "CO2 MAX": "mean",
    "WS": "mean",
    "WS MIN": "mean",
    "WS MAX gust": "mean",
    "WD": "mean",
    "WD MIN": "mean",
    "WD MAX gust": "mean",
    "UBat": "mean",
    "UBat MIN": "mean",
    "UBat MAX": "mean",
}

UNITS = {
    "AirTemperature": "°C",
    "Precipitation": "mm",
    "Humidity": "%",
    "Radiation": "W/m²",
    "CO2": "ppm",
    "WS": "m/s",
    "WD": "°",
    "UBat": "V",
}
